use std::fmt::Error;

use cassandra_cpp::{stmt, CassResult, Session};

use crate::model::user::User;

use super::KEYSPACE_NAME;
pub struct UserRepository<'a> {
    session: &'a Session,
    table: String,
}

impl<'a> UserRepository<'a> {
    pub fn new(session: &'a Session) -> UserRepository<'a> {
        let table = format!("{KEYSPACE_NAME}.user");
        let query = stmt!(&format!(
            "CREATE TABLE IF NOT EXISTS {table} (
                id uuid,
                login text,
                hash ascii,
                salt text,
                PRIMARY KEY (id)
            );"
        ));

        session
            .execute(&query)
            .wait()
            .expect("Should create user keyspace if not exists");

        UserRepository { session, table }
    }

    pub fn create(&self, user: &User) -> Result<(), cassandra_cpp::Error> {
        let mut query = stmt!(&format!(
            "INSERT INTO {} (id, login, hash, salt)
             VALUES (?, ?, ?, ?);", self.table
        ));
        let uuid = cassandra_cpp::Uuid::from(user.id().clone());
        query.bind_uuid(0, uuid).expect("Binds the id");
        query.bind_string(1, user.login()).expect("Binds the login");
        query.bind_string(2, user.hash()).expect("Binds the hash");
        query.bind_string(3, user.salt()).expect("Binds the salt");
        self.session.execute(&query).wait()?;
        Ok(())
    }

    pub fn read_all(&self) -> Result<CassResult, cassandra_cpp::Error> {
        let query = stmt!(&format!("SELECT * FROM {};", self.table));
        let res = self.session.execute(&query).wait()?;
        Ok(res)
    }
}
